/*:
 * @author Casper Gaming
 * @url https://www.caspergaming.com/plugins/cgmz/battlecommandwindow/
 * @target MZ
 * @base CGMZ_Core
 * @orderAfter CGMZ_Core
 * @plugindesc Allows you to change the commands on battle windows
 * @help
 * ============================================================================
 * For terms and conditions using this plugin in your game please visit:
 * https://www.caspergaming.com/terms-of-use/
 * ============================================================================
 * Become a Patron to get access to beta/alpha plugins plus other goodies!
 * https://www.patreon.com/CasperGamingRPGM
 * ============================================================================
 * Version: Beta
 * ----------------------------------------------------------------------------
 * Compatibility: Only tested with my CGMZ plugins.
 * Made for RPG Maker MZ 1.7.0
 * ----------------------------------------------------------------------------
 * Description: Change the party or actor command window in battle. Party
 * window is the one that includes fight or escape options. Actor command
 * window is the one that includes guard, skill, item, etc. by default.
 * ----------------------------------------------------------------------------
 * Documentation:
 * ------------------------------Beta Notes------------------------------------
 * Want additional features not already present/listed above? Make suggestions
 * on the Patreon Post or in my discord under the #suggestions channel!
 * https://discord.gg/Gbx7JXP
 * ----------------------------Plugin Commands---------------------------------
 * This plugin does not have any plugin commands.
 * -------------------------------Conflicts------------------------------------
 * This plugin will overwrite the default window commands if keep originals is
 * off. It is best to place this below any other plugins that add commands to
 * the battle command windows if this option is used.
 * ----------------------------Command Symbol----------------------------------
 * The command symbol should be unique and not blank for every command. This
 * symbol is how the plugin knows internally which JS code to run.
 * ---------------------------Original Symbols---------------------------------
 * Some Command Symbols can have special meanings, mainly when they represent
 * the original commands.
 * 
 * The following symbols represent the original party commands:
 * fight - Will handle like the original fight command
 * escape - Will handle like the original escape command
 * 
 * The following symbols represent the original actor commands:
 * attack - Will handle like the original attack command
 * skill - Will handle like the original skill command(s)
 * item - Will handle like the original item command
 * guard - Will handle like the original guard command
 * 
 * It is important that you do not use these strings as the Command Symbol
 * property unless you mean to refer to the original commands.
 * ------------------------------Saved Games-----------------------------------
 * This plugin is fully compatible with saved games.
 *
 * ✓ Add this plugin to a saved game and it will work as expected
 * ✓ Change any plugin params and changes will be reflected in saved games
 * ✓ Remove the plugin with no issue to save data
 * -----------------------------Filename---------------------------------------
 * The filename for this plugin MUST remain CGMZ_BattleCommandWindow.js
 * This is what it comes as when downloaded. The filename is used to load
 * parameters and execute plugin commands. If you change it, things will begin
 * behaving incorrectly and your game will probably crash. Please do not
 * rename the js file.
 * -------------------------Version History------------------------------------
 * Version 1.0.0 - Initial Release
 *
 * @param Party Command Alignment
 * @type select
 * @option left
 * @option center
 * @option right
 * @default center
 * @desc The alignment of the command text in the party command window
 *
 * @param Actor Command Alignment
 * @type select
 * @option left
 * @option center
 * @option right
 * @default center
 * @desc The alignment of the command text in the actor command window
 *
 * @param Keep Original Commands Party
 * @type boolean
 * @default true
 * @desc Determine whether to show the original party commands in their original order.
 *
 * @param Keep Original Commands Actor
 * @type boolean
 * @default true
 * @desc Determine whether to show the original actor commands in their original order.
 *
 * @param Enable Text Codes
 * @type boolean
 * @default true
 * @desc Allow the use of text codes (such as colors) ?
 *
 * @param Actor Commands
 * @type struct<ActorHandler>[]
 * @desc Command Name and associated js commands for actors
 * @default []
 *
 * @param Party Commands
 * @type struct<Handler>[]
 * @desc Command Name and associated js commands for party
 * @default []
*/
/*~struct~Handler:
 * @param Command Name
 * @desc Name of the command to display in the command window.
 *
 * @param Command Symbol
 * @desc This symbol is used internally to recognize the command.
 * Special meaning for original commands (see documentation).
 *
 * @param JS Command
 * @type note
 * @desc JavaScript to run when command is selected.
 * @default ""
 *
 * @param JS Condition
 * @type note
 * @desc JavaScript conditional to check to enable the command
 * @default "true"
 *
 * @param JS Show Condition
 * @type note
 * @desc JavaScript conditional to check to show the command
 * @default "true"
*/
/*~struct~ActorHandler:
 * @param Actor
 * @type actor
 * @desc The actor to display the command for
 *
 * @param Command Name
 * @desc Name of the command to display in the command window.
 *
 * @param Command Symbol
 * @desc This symbol is used internally to recognize the command.
 * Special meaning for original commands (see documentation).
 *
 * @param JS Command
 * @type note
 * @desc JavaScript to run when command is selected.
 * @default ""
 *
 * @param JS Condition
 * @type note
 * @desc JavaScript conditional to check to enable the command
 * @default "true"
 *
 * @param JS Show Condition
 * @type note
 * @desc JavaScript conditional to check to show the command
 * @default "true"
*/
var Imported = Imported || {};
Imported.CGMZ_BattleCommandWindow = true;
var CGMZ = CGMZ || {};
CGMZ.Versions = CGMZ.Versions || {};
CGMZ.Versions["Battle Command Window"] = "Beta";
CGMZ.BattleCommandWindow = {};
CGMZ.BattleCommandWindow.parameters = PluginManager.parameters('CGMZ_BattleCommandWindow');
CGMZ.BattleCommandWindow.PartyCommandAlignment = CGMZ.BattleCommandWindow.parameters["Party Command Alignment"];
CGMZ.BattleCommandWindow.ActorCommandAlignment = CGMZ.BattleCommandWindow.parameters["Actor Command Alignment"];
CGMZ.BattleCommandWindow.KeepOriginalsParty = (CGMZ.BattleCommandWindow.parameters["Keep Original Commands Party"] === "true");
CGMZ.BattleCommandWindow.KeepOriginalsActor = (CGMZ.BattleCommandWindow.parameters["Keep Original Commands Actor"] === "true");
CGMZ.BattleCommandWindow.EnableTextCodes = (CGMZ.BattleCommandWindow.parameters["Enable Text Codes"] === "true");
CGMZ.BattleCommandWindow.PartyCommandsArray = CGMZ_Utils.parseJSON(CGMZ.BattleCommandWindow.parameters["Party Commands"], [], "CGMZ Battle Command Window", "Your Party Commands parameter could not be parsed. No custom party commands will be loaded.");
CGMZ.BattleCommandWindow.ActorCommandsArray = CGMZ_Utils.parseJSON(CGMZ.BattleCommandWindow.parameters["Actor Commands"], [], "CGMZ Battle Command Window", "Your Actor Commands parameter could not be parsed. No custom party commands will be loaded.");
CGMZ.BattleCommandWindow.PartyCommands = [];
CGMZ.BattleCommandWindow.ActorCommands = [];
for(let i = 0; i < CGMZ.BattleCommandWindow.PartyCommandsArray.length; i++) {
	const cmd = CGMZ_Utils.parseJSON(CGMZ.BattleCommandWindow.PartyCommandsArray[i], null, "CGMZ Battle Command Window", "One of your party commands was invalid. Skipping.")
	if(cmd) CGMZ.BattleCommandWindow.PartyCommands.push(cmd);
}
for(let i = 0; i < CGMZ.BattleCommandWindow.ActorCommandsArray.length; i++) {
	const cmd = CGMZ_Utils.parseJSON(CGMZ.BattleCommandWindow.ActorCommandsArray[i], null, "CGMZ Battle Command Window", "One of your actor commands was invalid. Skipping.")
	if(cmd) CGMZ.BattleCommandWindow.ActorCommands.push(cmd);
}
//=============================================================================
// Scene Battle
//-----------------------------------------------------------------------------
// Handling for command window entries
//=============================================================================
//-----------------------------------------------------------------------------
// Handling for custom Party Commands added through the plugin
//-----------------------------------------------------------------------------
Scene_Battle.prototype.CGMZ_BattleCommandWindow_commandPartyCustom = function() {
	for(let i = 0; i < CGMZ.BattleCommandWindow.PartyCommands.length; i++) {
		if(this._partyCommandWindow.currentSymbol() === CGMZ.BattleCommandWindow.PartyCommands[i]["Command Symbol"]) {
			try {
				const cmdJSON = CGMZ.BattleCommandWindow.PartyCommands[i]["JS Command"];
				const command = CGMZ_Utils.parseJSON(cmdJSON, "this._partyCommandWindow.setup()", "CGMZ Battle Command Window", "Your JS Command could not be parsed, it will do nothing.");
				eval(command);
			}
			catch (e) {
				const origin = "CGMZ Battle Command Window";
				const suggestion = "Check your JavaScript party commands";
				$cgmzTemp.reportError(e.message, origin, suggestion);
			}
		}
	}
};
//-----------------------------------------------------------------------------
// Handling for custom Actor Commands added through the plugin
//-----------------------------------------------------------------------------
Scene_Battle.prototype.CGMZ_BattleCommandWindow_commandActorCustom = function() {
	for(let i = 0; i < CGMZ.BattleCommandWindow.ActorCommands.length; i++) {
		if(this._actorCommandWindow.currentSymbol() === CGMZ.BattleCommandWindow.ActorCommands[i]["Command Symbol"]) {
			try {
				const cmdJSON = CGMZ.BattleCommandWindow.ActorCommands[i]["JS Command"];
				const command = CGMZ_Utils.parseJSON(cmdJSON, "this._actorCommandWindow.activate()", "CGMZ Battle Command Window", "Your JS Command could not be parsed, it will do nothing.");
				eval(command);
			}
			catch (e) {
				const origin = "CGMZ Battle Command Window";
				const suggestion = "Check your JavaScript party commands";
				$cgmzTemp.reportError(e.message, origin, suggestion);
			}
		}
	}
};
//-----------------------------------------------------------------------------
// Alias. Add additional party commands.
//-----------------------------------------------------------------------------
const alias_CGMZ_BattleCommandWindow_createPartyCommandWindow = Scene_Battle.prototype.createPartyCommandWindow;
Scene_Battle.prototype.createPartyCommandWindow = function() {
	alias_CGMZ_BattleCommandWindow_createPartyCommandWindow.call(this);
	for(const cmd of CGMZ.BattleCommandWindow.PartyCommands) {
		if(this.CGMZ_BattleCommandWindow_isCustomCommand(cmd["Command Symbol"])) {
			this._partyCommandWindow.setHandler(cmd["Command Symbol"], this.CGMZ_BattleCommandWindow_commandPartyCustom.bind(this));
		}
	}
};
//-----------------------------------------------------------------------------
// Alias. Add additional actor commands.
//-----------------------------------------------------------------------------
const alias_CGMZ_BattleCommandWindow_createActorCommandWindow = Scene_Battle.prototype.createActorCommandWindow;
Scene_Battle.prototype.createActorCommandWindow = function() {
    alias_CGMZ_BattleCommandWindow_createActorCommandWindow.call(this);
	for(const cmd of CGMZ.BattleCommandWindow.ActorCommands) {
		if(this.CGMZ_BattleCommandWindow_isCustomCommand(cmd["Command Symbol"])) {
			this._actorCommandWindow.setHandler(cmd["Command Symbol"], this.CGMZ_BattleCommandWindow_commandActorCustom.bind(this));
		}
	}
};
//-----------------------------------------------------------------------------
// Determine if command is a custom command in need of custom handler
//-----------------------------------------------------------------------------
Scene_Battle.prototype.CGMZ_BattleCommandWindow_isCustomCommand = function(symbol) {
	const defaultCommands = ['fight', 'escape', 'attack', 'skill', 'guard', 'item'];
	return !defaultCommands.includes(symbol);
};
//=============================================================================
// Window Party Command
//-----------------------------------------------------------------------------
// Change commands in the command window
//=============================================================================
//-----------------------------------------------------------------------------
// Alias. Add original commands in original order if user wishes
//-----------------------------------------------------------------------------
const alias_CGMZ_BattleCommandWindow_PartyCommand_makeCommandList = Window_PartyCommand.prototype.makeCommandList;
Window_PartyCommand.prototype.makeCommandList = function() {
	if(CGMZ.BattleCommandWindow.KeepOriginalsParty) {
		alias_CGMZ_BattleCommandWindow_PartyCommand_makeCommandList.call(this);
	}
	for(const cmd of CGMZ.BattleCommandWindow.PartyCommands) {
		const showCondition = CGMZ_Utils.parseJSON(cmd["JS Show Condition"], "true", "CGMZ Battle Command Window", "Your JS Show Condition could not be parsed, it will be treated as true for command: " + cmd["Command Symbol"]);
		if(!!eval(showCondition)) {
			const condition = CGMZ_Utils.parseJSON(cmd["JS Condition"], "true", "CGMZ Battle Command Window", "Your JS Condition could not be parsed, it will be treated as true for command: " + cmd["Command Symbol"]);
			this.addCommand(cmd["Command Name"], cmd["Command Symbol"], !!eval(condition));
		}
	}
};
//-----------------------------------------------------------------------------
// Change alignment of command text
//-----------------------------------------------------------------------------
Window_PartyCommand.prototype.itemTextAlign = function() {
    return CGMZ.BattleCommandWindow.PartyCommandAlignment;
};
//-----------------------------------------------------------------------------
// Allow use of text codes in command
//-----------------------------------------------------------------------------
Window_PartyCommand.prototype.drawItem = function(index) {
	if(CGMZ.BattleCommandWindow.EnableTextCodes) {
		const rect = this.itemLineRect(index);
		const align = this.itemTextAlign();
		this.resetTextColor();
		this.changePaintOpacity(this.isCommandEnabled(index));
		this.CGMZ_drawTextLine(this.commandName(index), rect.x, rect.y, rect.width, align);
	} else {
		Window_Command.prototype.drawItem.call(this, index);
	}
};
//=============================================================================
// Window Actor Command
//-----------------------------------------------------------------------------
// Change commands in the command window
//=============================================================================
//-----------------------------------------------------------------------------
// Alias. Add original commands in original order if user wishes
//-----------------------------------------------------------------------------
const alias_CGMZ_BattleCommandWindow_ActorCommand_makeCommandList = Window_ActorCommand.prototype.makeCommandList;
Window_ActorCommand.prototype.makeCommandList = function() {
	if(CGMZ.BattleCommandWindow.KeepOriginalsActor) {
		alias_CGMZ_BattleCommandWindow_ActorCommand_makeCommandList.call(this);
	}
	if(!this._actor) return;
	for(const cmd of CGMZ.BattleCommandWindow.ActorCommands) {
		if(this.CGMZ_isDefaultCommand(cmd["Command Symbol"])) {
			switch(cmd["Command Symbol"]) {
				case 'attack': this.addAttackCommand(); break;
				case 'skill': this.addSkillCommands(); break;
				case 'item': this.addItemCommand(); break;
				case 'guard': this.addGuardCommand(); break;
			}
		} else {
			if(this._actor._actorId === Number(cmd["Actor"])) {
				const showCondition = CGMZ_Utils.parseJSON(cmd["JS Show Condition"], "true", "CGMZ Battle Command Window", "Your JS Show Condition could not be parsed, it will be treated as true for command: " + cmd["Command Symbol"]);
				if(!!eval(showCondition)) {
					const condition = CGMZ_Utils.parseJSON(cmd["JS Condition"], "true", "CGMZ Battle Command Window", "Your JS Condition could not be parsed, it will be treated as true for command: " + cmd["Command Symbol"]);
					this.addCommand(cmd["Command Name"], cmd["Command Symbol"], !!eval(condition));
				}
			}
		}
	}
};
//-----------------------------------------------------------------------------
// Is Default Command ?
//-----------------------------------------------------------------------------
Window_ActorCommand.prototype.CGMZ_isDefaultCommand = function(symbol) {
    return (symbol === 'attack' || symbol === 'skill' || symbol === 'item' || symbol === 'guard');
};
//-----------------------------------------------------------------------------
// Change alignment of command text
//-----------------------------------------------------------------------------
Window_ActorCommand.prototype.itemTextAlign = function() {
    return CGMZ.BattleCommandWindow.ActorCommandAlignment;
};
//-----------------------------------------------------------------------------
// Allow use of text codes in command
//-----------------------------------------------------------------------------
Window_ActorCommand.prototype.drawItem = function(index) {
	if(CGMZ.BattleCommandWindow.EnableTextCodes) {
		const rect = this.itemLineRect(index);
		const align = this.itemTextAlign();
		this.resetTextColor();
		this.changePaintOpacity(this.isCommandEnabled(index));
		this.CGMZ_drawTextLine(this.commandName(index), rect.x, rect.y, rect.width, align);
	} else {
		Window_Command.prototype.drawItem.call(this, index);
	}
};